<?php

require_once dirname(dirname(__FILE__)) . '/include/etc/config.php';
require_once dirname(dirname(__FILE__)) . '/include/grouper.php';

$iterations = 100;
$uuid = 'SURFNET.hansz';
$grouper['stem'] = 'test';
$decimals = 3;

$nominal = array(
	'connect'			=> array(0.25, 0.30),
	'create'			=> array(0.09, 0.25),
	'addmember'			=> array(0.04, 0.15),
	'delmember'			=> array(0.04, 0.35),
	'setpriv'			=> array(0.13, 0.30),
	'getpriv'			=> array(0.05, 0.15),
	'groups'			=> array(0.04, 0.15),
	'members'			=> array(0.03, 0.15),
	'find'      		=> array(0.015, 0.15),
	'delete'    		=> array(0.13, 0.30),
);

function pm_start($name) {
	global $decimals;
	$now = microtime(true);
	return array(
		'name' => $name,
		'start' => $now,
		'last' => $now,
		'laps' => 0,
		'decimals' => $decimals,
	);
}

function pm_lap(&$pm) {
	$pm['laps'] += 1;
	$now = microtime(true);
	$delta = round($now - $pm['last'], $pm['decimals']);
	if ($pm['laps'] == 1) {
		$pm['min'] = $delta;
		$pm['max'] = $delta;
	} else if ($delta < $pm['min']) {
		$pm['min'] = $delta;
	} else if ($delta > $pm['max']) {
		$pm['max'] = $delta;
	}
	$pm['last'] = $now;
}

function pm_decimals(&$pm, $f) {
	return sprintf("%." . $pm['decimals'] . 'f', $f);	
}

function pm_end(&$pm, $n) {
	global $nominal;
	$pm['end'] = microtime(true);
	$delta = round(microtime(true) - $pm['start'], $pm['decimals']);
	$average = round($delta/$pm['laps'], $pm['decimals']);
	print $pm['name'] . ":\n";
	print ' ' . $pm['laps'] . 'x in ' . pm_decimals($pm, $delta) . ' seconds, ';
	print 'average=' .  pm_decimals($pm, $average) . ', range=[' .  pm_decimals($pm, $pm['min']) . '-' .  pm_decimals($pm, $pm['max']) . '], nominal=[' .  pm_decimals($pm, $nominal[$n][0]) . '-' .  pm_decimals($pm, $nominal[$n][1]) . ']';
	if ($pm['min'] < $nominal[$n][0]) print " ! ";
	if ($pm['max'] > $nominal[$n][1]) print " # ";
	print "\n";
}

$pm = pm_start('Grouper Connection');
$gr = new_grouper();
if (!$gr->isConnected()) {
	print ' SURFteams initialization failed!';
    exit;
}
pm_lap($pm);
pm_end($pm, 'connect');

$new_groups = array();
$pm = pm_start('Group Creation');
for ($i = 0; $i < $iterations; $i += 1) {
	$new_name = 'test_' . $i;
	$result = $gr->addGroup($new_name, $new_name);
	if ($result == -1) {
		print 'Group creation failed: ' . $new_name . "\n";
	}
	$new_groups[] = $grouper['stem'] . ':' . $new_name;
	pm_lap($pm);
}
pm_end($pm, 'create');

$pm = pm_start('Add Member');
foreach ($new_groups as $group_id) {
	if ($gr->addMember($group_id, $uuid, $uuid) == FALSE) {
		print 'Add member failed: ' . $group_id . " - " . $uuid . "\n";
	}
	pm_lap($pm);
}
pm_end($pm, 'addmember');

$pm = pm_start('Delete Member');
foreach ($new_groups as $group_id) {
	if ($gr->deleteMember($uuid, $group_id) == FALSE) {
		print 'Delete member failed: ' . $group_id . " - " . $uuid . "\n";
	}
	pm_lap($pm);
}
pm_end($pm, 'delmember');

$pm = pm_start('Set Admin Privileges (3x)');
foreach ($new_groups as $group_id) {
	if ($gr->setPrivilege($group_id, $uuid, 'admin', TRUE) == FALSE) {
		print 'Set admin privilege failed: ' . $group_id . " - " . $uuid . "\n";
	}
	if ($gr->setPrivilege($group_id, $uuid, 'update', TRUE) == FALSE) {
		print 'Set update privilege failed: ' . $group_id . " - " . $uuid . "\n";
	}
	if ($gr->setPrivilege($group_id, $uuid, 'optout', TRUE) == FALSE) {
		print 'Set optout privilege failed: ' . $group_id . " - " . $uuid . "\n";
	}
	pm_lap($pm);
}
pm_end($pm, 'setpriv');

$pm = pm_start('Get Subject Privileges');
foreach ($new_groups as $group_id) {
	if (!is_array($gr->getSubjectGroupPrivileges($uuid, $group_id))) {
		print 'Get subject privileges failed: ' . $group_id . " - " . $uuid . "\n";
	}
	pm_lap($pm);
}
pm_end($pm, 'getpriv');

// NB: more iterations means more groups, means longer messages...
$pm = pm_start('Get Subject Groups');
for ($i = 0; $i < $iterations; $i += 1) {
	if ($gr->getSubjectGroups($uuid) == -1) {
		print "Get subject groups failed!\n";
	}
	pm_lap($pm);
}
pm_end($pm, 'groups');

$pm = pm_start('Get Group Members');
foreach ($new_groups as $key => $group_id) {
	if ($gr->getGroupMembers($group_id) == -1) {
		print "Get group members failed!\n";
	}
	pm_lap($pm);
}
pm_end($pm, 'members');

// NB: more iterations means more groups, means longer messages...
$pm = pm_start('Find Groups');
for ($i = 0; $i < $iterations; $i += 1) {
	if ($gr->findGroups(rand(0,9)) == -1) {
		print "Find groups failed!\n";
	}
	pm_lap($pm);
}
pm_end($pm, 'find');

$pm = pm_start('Group Delete');
foreach ($new_groups as $key => $group_id) {
	if ($gr->deleteGroup($group_id)) unset($new_groups[$key]);
	pm_lap($pm);
}
pm_end($pm, 'delete');

if (count($new_groups) > 0) {
	print 'Testgroups that were created and should have been deleted:' . "\n";
	print_r($new_groups);
}

?>
